@@ -15,9 +15,13 @@ module Statistics.Constants
     , m_huge
     , m_1_sqrt_2
     , m_2_sqrt_pi
+    , m_ln_sqrt_2_pi
     , m_max_exp
     , m_sqrt_2
     , m_sqrt_2_pi
+    , m_pos_inf
+    , m_neg_inf
+    , m_NaN
     ) where
 
 -- | A very large number.
@@ -50,7 +54,27 @@ m_1_sqrt_2 :: Double
 m_1_sqrt_2 = 0.7071067811865475244008443621048490392848359376884740365883
 {-# INLINE m_1_sqrt_2 #-}
 
--- | The smallest 'Double' larger than 1.
+-- | The smallest 'Double' &#949; such that 1 + &#949; &#8800; 1.
 m_epsilon :: Double
 m_epsilon = encodeFloat (signif+1) expo - 1.0
     where (signif,expo) = decodeFloat (1.0::Double)
+
+-- | @log(sqrt((2*pi)) / 2@
+m_ln_sqrt_2_pi :: Double
+m_ln_sqrt_2_pi = 0.9189385332046727417803297364056176398613974736377834128171
+{-# INLINE m_ln_sqrt_2_pi #-}
+
+-- | Positive infinity.
+m_pos_inf :: Double
+m_pos_inf = 1/0
+{-# INLINE m_pos_inf #-}
+
+-- | Negative infinity.
+m_neg_inf :: Double
+m_neg_inf = -1/0
+{-# INLINE m_neg_inf #-}
+
+-- | Not a number.
+m_NaN :: Double
+m_NaN = 0/0
+{-# INLINE m_NaN #-}
