@@ -20,7 +20,7 @@ module Statistics.Distribution.Hypergeometric
     (
       HypergeometricDistribution
     -- * Constructors
-    , fromParams
+    , hypergeometric
     -- ** Accessors
     , hdM
     , hdL
@@ -28,10 +28,8 @@ module Statistics.Distribution.Hypergeometric
     ) where
 
 import Control.Exception (assert)
-import qualified Data.Vector.Unboxed as U
 import Data.Typeable (Typeable)
-import Statistics.Math (choose, logFactorial)
-import Statistics.Constants (m_max_exp)
+import Statistics.Math (choose)
 import qualified Statistics.Distribution as D
 
 data HypergeometricDistribution = HD {
@@ -41,9 +39,10 @@ data HypergeometricDistribution = HD {
     } deriving (Eq, Read, Show, Typeable)
 
 instance D.Distribution HypergeometricDistribution where
-    density    = density
-    cumulative = cumulative
-    quantile   = quantile
+    cumulative d x = D.sumProbabilities d 0 (floor x)
+
+instance D.DiscreteDistr HypergeometricDistribution where
+    probability = probability
 
 instance D.Variance HypergeometricDistribution where
     variance = variance
@@ -63,45 +62,21 @@ mean :: HypergeometricDistribution -> Double
 mean (HD m l k) = fromIntegral k * fromIntegral m / fromIntegral l
 {-# INLINE mean #-}
 
-fromParams :: Int               -- ^ /m/
-           -> Int               -- ^ /l/
-           -> Int               -- ^ /k/
-           -> HypergeometricDistribution
-fromParams m l k =
-    assert (m > 0 && m <= l) .
+hypergeometric :: Int               -- ^ /m/
+               -> Int               -- ^ /l/
+               -> Int               -- ^ /k/
+               -> HypergeometricDistribution
+hypergeometric m l k =
+    assert (m >= 0 && m <= l) .
     assert (l > 0) .
     assert (k > 0 && k <= l) $
     HD m l k
-{-# INLINE fromParams #-}
+{-# INLINE hypergeometric #-}
 
-density :: HypergeometricDistribution -> Double -> Double
-density (HD mi li ki) x
-    | l <= 70    = (mi <> xi) * ((li - mi) <> (ki - xi)) / (li <> ki)
-    | r > maxVal = 1/0
-    | otherwise  = exp r
-  where
-    a <> b = a `choose` b
-    r = f m + f (l-m) - f l - f xi - f (k-xi) + f k -
-        f (m-xi) - f (l-m-k+xi) + f (l-k)
-    f = logFactorial
-    maxVal = fromIntegral (m_max_exp - 1) * log 2
-    xi = floor x
-    m = fromIntegral mi
-    l = fromIntegral li
-    k = fromIntegral ki
-{-# INLINE density #-}
-
-cumulative :: HypergeometricDistribution -> Double -> Double
-cumulative d@(HD m l k) x
-    | x < fromIntegral imin  = 0
-    | x >= fromIntegral imax = 1
-    | otherwise = min r 1
-  where
-    imin = max 0 (k - l + m)
-    imax = min k m
-    r = U.sum . U.map (density d . fromIntegral) . U.enumFromTo imin . floor $ x
-{-# INLINE cumulative #-}
-
-quantile :: HypergeometricDistribution -> Double -> Double
-quantile = error "Statistics.Distribution.Hypergeometric.quantile: not yet implemented"
-{-# INLINE quantile #-}
+-- Naive implementation
+probability :: HypergeometricDistribution -> Int -> Double
+probability (HD mi li ki) n
+  | n < max 0 (mi+ki-li) || n > min mi ki = 0
+  | otherwise =
+      choose mi n * choose (li - mi) (ki - n) / choose li ki
+{-# INLINE probability #-}
