@@ -17,47 +17,44 @@ module Statistics.Distribution.Poisson
     (
       PoissonDistribution
     -- * Constructors
-    , fromLambda
-    -- , fromSample
+    , poisson
+    -- * Accessors
+    , poissonLambda
     ) where
 
 import Data.Typeable (Typeable)
-import qualified Data.Vector.Unboxed as U
 import qualified Statistics.Distribution as D
-import Statistics.Constants (m_huge)
-import Statistics.Math (logGamma)
+import Statistics.Math (logGamma, factorial)
 
 newtype PoissonDistribution = PD {
-      pdLambda :: Double
+      poissonLambda :: Double
     } deriving (Eq, Read, Show, Typeable)
 
 instance D.Distribution PoissonDistribution where
-    density    = density
-    cumulative = cumulative
-    quantile   = quantile
+    cumulative d x = D.sumProbabilities d 0 (floor x)
+    {-# INLINE cumulative #-}
+
+instance D.DiscreteDistr PoissonDistribution where
+    probability = probability
 
 instance D.Variance PoissonDistribution where
-    variance = pdLambda
+    variance = poissonLambda
     {-# INLINE variance #-}
 
 instance D.Mean PoissonDistribution where
-    mean = pdLambda
+    mean = poissonLambda
     {-# INLINE mean #-}
 
-fromLambda :: Double -> PoissonDistribution
-fromLambda = PD
-{-# INLINE fromLambda #-}
+-- | Create po
+poisson :: Double -> PoissonDistribution
+poisson = PD
+{-# INLINE poisson #-}
 
-density :: PoissonDistribution -> Double -> Double
-density (PD l) x = exp (x * log l - l - logGamma x)
-{-# INLINE density #-}
-
-cumulative :: PoissonDistribution -> Double -> Double
-cumulative d = U.sum . U.map (density d . fromIntegral) .
-               U.enumFromTo (0::Int) . floor
-{-# INLINE cumulative #-}
-
-quantile :: PoissonDistribution -> Double -> Double
-quantile d p = fromIntegral . r $ D.findRoot d p (pdLambda d) 0 m_huge
-    where r = round :: Double -> Int
-{-# INLINE quantile #-}
+probability :: PoissonDistribution -> Int -> Double
+probability (PD l) n
+  | n < 0                   = 0
+  | l < 20 && n <= 100      = exp (-l) * l ** x / factorial n
+  | otherwise               = exp (x * log l - logGamma (x + 1) - l)
+    where
+      x = fromIntegral n
+{-# INLINE probability #-}
